'DL165   Greenhouse Controller

'THIS IS THE PROGRAM FOR THE AT90S8535 uC LOCATED IN THE GREENHOUSE
'IT GATHERS THE SENSOR DATA AND OPERATES THE VENTS/FAN/HEATER ACCORDINGLY
'   BASED ON TOP, BOT, BATT AND THE POT VALUES
'   THEN XMITS THE DATA (1200 BAUD) AT 433.9 MHz TO THE RECEIVER,
'   DISPLAYS THE SAME DATA ON THE LCD IN THE GREENHOUSE,
'   WAITS 10 SECONDS AND THEN REPEATS THE ABOVE

'IT MEASURES TOP=XX (GH TOP TEMP), BOT=XX (GH BOTTOM TEMP, OUT=XX (OUTSIDE TEMP),
'  M=L/H (SOIL MOISTURE LOW/HIGH), BATT=XX.X (BATTERY VOLTAGE, MAX IS 18.8V)
'  AND VENTS OPEN POT AND HEATER OFF POT

'THE VENTS WILL OPEN IF TOP >= VENTS OPEN AND SHUT IF TOP <= (VENTS OPEN - 15)
'THE VENTS WILL NOT OPEN IF THE HEATER IS ON
'THE FAN WILL TURN ON AT (VENTS OPEN + 10) AND OFF AT (VENTS OPEN + 5)

'THE HEATER WILL TURN ON IF BOT <= HEATER ON AND OFF AT (HEATER ON + 10)
'    THE HEATER WILL NOT TURN ON IF VENTS ARE OPEN OR FAN IS ON

'THE BUZZER AT THE RECEIVER WILL TURN ON IF
'   TOP <= 32 OR >=110
'   BOT <= 32 OR > 99
'   BATT < 9.1

'VENTS=255 OR 0 (OPEN/SHUT), MOISTURE=255 OR 0 (LO/HI)
'FAN = 0 OR 255 (OFF/ON), HEATER = 0 OR 255 (OFF/ON) BATTERY=XX.X


'STEPPER MOTOR = CyberPak SM-101-DS, 1.3A/ph 2.5ohm/ph
'Vref=2.5v, Rs=1.0 ohm, Itrip(max)=Vref/3Rs = 2.5v/3.0 = 0.833 amps max
'DAC data for ALLEGRO "3955" motor controller chip
'D2 D1 D0
'H  L  L   70.7% of Itrip = 0.589 amps
'L  H  L   38.2% of Itrip = 0.318 amps
'L  L  H   19.5% of Itrip = 0.163 amps
'L  L  L   ZERO CURRENT TO MOTOR - standby mode
'RED => #15, YELLOW => #9 0f 3955"B"   Phase(Pb) => PC0 #22 of 8535
'ORANGE => #15, BROWN => #9 0f 3955"A" Phase(Pa) => PC1 #23 of 8535

'Brevel Motor #713 10 ohm coil, I=200ma @ 12v, 8revs/15sec, 4.5 rev=open or shut
'Vref=2.5v, Rs=1.0 ohm, Itrip(max)=Vref/3Rs = 2.5v/3.0 = 0.833 amps max
'for D2 HI then Itrip = 0.707(0.278ma) = 589ma
'+12v on "+" terminal of motor => OPEN (vents open)

$regfile = "8535def.dat"
$crystal = 7372800
$baud = 9600

Dim Junk As Byte
Dim Synch As Byte
Dim Ttop As Byte
Dim Tbot As Byte
Dim Tout As Byte
Dim Ventsopen As Byte
Dim Heateroffventshut As Byte
'DL165   Greenhouse Controller

Dim Heater_on As Byte
Dim Heat As Byte
Dim Vents As Byte
Dim Ventshut As Byte
Dim Moisture As Byte
Dim Fan As Byte
Dim Fan_on As Byte
Dim Fan_off As Byte
Dim Batt As Byte
Dim Battd10 As Byte
Dim Battr As Byte
Dim Message As Byte
Dim Tx As Byte
Dim N As Byte
Dim Temp_var As Single
Dim Tempword As Word
' 2.5/1023*(1degF/10mv)=0.2443792766373
Const Kt = 0.2443792766373
'Battery divider= +12v  56K  Batt  10K  -12v(gnd)
Const Kb = 0.1593352883675


'Define pin direction for ADC input pins ADC0 thru ADC7 pins #40 to #33 resp.
Ddra = &B00000000
Porta = 0       'set porta.0 thru porta.7 as inputs with no pull up resistor

Config Portc = &B11111111
Pa Alias Portc.1       'C1(#23) is Pa (PHASE A)
Pb Alias Portc.0       'C0(#22) is Pb (PHASE B)
D2 Alias Portc.2       'C2(#24) is D2 to the bridge DAC

Phgear Alias Portd.5
D2gear Alias Portd.4

'Portc.3      'C3(#25) is the output that turns on the FAN
Portc.4 = 1       'C4(#26) is output to Heater ON

Config Portd = &B11111110
Portd.6 = 1       'D6(#20) is the data out (RS232 format) to the XMIT #2
Portd.5 = 1       'D5(#19) is phase(direction) signal to the motor bd. for gear motor
Portd.4 = 1       'D4(#18) is D2 signal to the motor board for the gear motor
'NOTE - there is a green LED w/internal resistor on XMIT PIN #2, lites when xmitting

Ddrb = &B11111111       'Portb pins are outputs
Config Portb = Output
 'B0(1) is RS to LCD
 'B1(2) is EN to LCD
 'B2(3) is DB4 to LCD
 'B3(4) is DB5 to LCD
 'B4(5) is DB6 to LCD
 'B5(6) is DB7 to LCD
Config Lcdpin = Pin , Db4 = Portb.2 , Db5 = Portb.3 , Db6 = Portb.4 , Db7 = Portb.5 , E = Portb.1 , Rs = Portb.0
Config Lcd = 20 * 4
Config Lcdmode = Port       'the report will show the settings
'DL165   Greenhouse Controller

Config Lcdbus = 4       '4 bit mode
Cursor Off       'HIDE THE LCD CURSOR
Cls       'CLEAR THE LCD DISPLAY

Junk = 126
Synch = 127



Config Adc = Single , Prescaler = Auto
Start Adc : Waitms 1       'POWER ON TO THE ADC, WAIT 1 milli second

Reset Portc.3       'PROGRAM STARTS WITH FAN OFF
Reset Portc.4       'PROGRAM STARTS WITH HEATER OFF
Gosub Shutvents       'PROGRAM STARTS WITH VENTS CLOSED

Tx = 0       'Tx keeps count of the number of transmisions from 0 to 9

Start:
Reset D2       'ZERO CURRENT TO THE STEPPER MOTOR
Reset D2gear       'ZERO CURRENT TO THE GEAR MOTOR
Message = 0

Gosub Convert
Gosub Do_what
Gosub Xmit
Gosub Displaylcd

Wait 10       'wait 10 second
Goto Start

End

Convert:
'Temperature=#counts(Getadc(x)*(2.500v/1023)*(1degF/10mv)=0.2443792766373

Tempword = Getadc(0)
Temp_var = Tempword * Kt : Ttop = Temp_var : Temp_var = Temp_var - Ttop
If Temp_var >= 0.5 Then Incr Ttop

Tempword = Getadc(1)
Temp_var = Tempword * Kt : Tbot = Temp_var : Temp_var = Temp_var - Tbot
If Temp_var >= 0.5 Then Incr Tbot
'************ setting Kt=0.239 during calibration for Tout ******************
Tempword = Getadc(2)
Temp_var = Tempword * 0.239 : Tout = Temp_var : Temp_var = Temp_var - Tout
If Temp_var >= 0.5 Then Incr Tout


'Moisture levels=LO or HI
'RU=1MEG ohm to +5v,if Rsoil is slightly < 1MEG ohm then Vadc<2.5v
'Note: for Vadc=2.50v then #counts=1023
Tempword = Getadc(3)
If Tempword >= 1020 Then Moisture = 255 Else Moisture = 0
'DL165   Greenhouse Controller

'Moisture = 255       'Moisture=LO
'Moisture = 0       'Moisture=HI

'Battery divider= +12v  56K  Batt  10K  -12v(gnd)   56K measures 55.2K
'Battery volt=(2.5v/1023)*counts*6.52*10 = counts* 0.1593352883675
'*10 is for 1 decimal place,if Batt=149(which is xmitted,it means battery=14.9v
Tempword = Getadc(4)
Temp_var = Tempword * Kb : Batt = Temp_var : Temp_var = Temp_var - Batt
If Temp_var >= 0.5 Then Incr Batt

Tempword = Getadc(6)       '2.5v/1023*100= 0.2443792766373
Temp_var = Tempword * 0.244 : Ventsopen = Temp_var
Temp_var = Temp_var - Ventsopen
If Temp_var >= 0.5 Then Incr Ventsopen

Tempword = Getadc(7)
Temp_var = Tempword * 0.244 : Heateroffventshut = Temp_var
Temp_var = Temp_var - Heateroffventshut


Return

'****************************************************************************
'****************************************************************************

Do_what:

'************************** TEMPERATURES *************************************
If Ttop <= 32 Or Ttop >= 110 Then Message = 255
If Tbot <= 32 Or Tbot >= 99 Then Message = 255

'************************** VENTS ********************************************
Ventshut = Ventsopen - 15

If Ttop >= Ventsopen And Vents = 0 And Heat=0 Then Gosub Openvents       'OPEN VENTS

If Ttop <= Ventshut And Vents = 255 Then Gosub Shutvents       'SHUT VENTS

'*************************** FAN *********************************************
Fan_on = Ventsopen + 10       'TURN FAN ON 10 DEG GREATER THEN VentsOpen
Fan_off = Ventsopen + 5       'TURN FAN OFF 5 DEG GREATER THEN VentsOpen

If Ttop >= Fan_on And Vents = 255 Then       '         Fan_on = Ventsopen + 10
Set Portc.3       'TURN ON THE FAN
Fan = 255
End If

If Ttop <= Fan_off Then       '        Fan_off = Ventsopen + 5
Reset Portc.3       'TURN OFF THE FAN
Fan = 0

End If



'DL165   Greenhouse Controller


'**************************** BATTERY ***************************************
If Batt < 91 Then Message = 255       'LOW VOLTAGE <9.1 volts
Battd10 = Batt / 10       '91/10=9
Battr = Batt Mod 10       'this equals remainder of 91/10=1

'***************************** HEATER ***************************************
Heater_on = Heateroffventshut - 10
If Tbot >= Heateroffventshut Then
Reset Portc.4       'Turn OFF Heater
Heat = 0
End If
' ********** Heater will only go ON if vents are SHUT and fan is OFF
If Tbot <= Heater_on And Vents = 0 And Fan = 0 Then
Set Portc.4       'Turn ON Heater
Heat = 255
End If
Return

'****************************************************************************
'****************************************************************************

Xmit:
Tx = Tx + 1
If Tx > 9 Then Tx = 0
Open "comd.6:1200,8,N,1,inverted" For Output As #1
Baud #1 , 1200
Print #1 , Junk : Waitms 25
Print #1 , Synch : Waitms 25
Print #1 , Ttop : Waitms 25
Print #1 , Tbot : Waitms 25
Print #1 , Tout : Waitms 25
Print #1 , Vents : Waitms 25
Print #1 , Ventshut : Waitms 25
Print #1 , Ventsopen : Waitms 25
Print #1 , Heateroffventshut : Waitms 25
Print #1 , Heater_on : Waitms 25
Print #1 , Heat : Waitms 25
Print #1 , Moisture : Waitms 25
Print #1 , Fan : Waitms 25
Print #1 , Battd10 : Waitms 25
Print #1 , Battr : Waitms 25
Print #1 , Message : Waitms 25
Print #1 , Tx : Waitms 25
Close #1
Return

'****************************************************************************
'************THIS IS THE DISPLAY FORMAT****************
'TOP=76  VENT60-100=S
'BOT=73  M=L  FAN=OFF
'OUT=69     BATT=12.3
'HTR=50-60  HEAT=OFFx

'DL165   Greenhouse Controller


Displaylcd:       'Display the same data xmitted on the LCD in the greenhouse
Cls
Locate 1 , 1 : Lcd "TOP=" ; Ttop       'select line 1 and position 1
Locate 1 , 9 : Lcd "VENT" ; Ventshut ; "-" ; Ventsopen ; "="
If Vents = 0 Then
Locate 1 , 20 : Lcd "S"
End If
If Vents = 255 Then
Locate 1 , 20 : Lcd "O"
End If

Locate 2 , 1 : Lcd "BOT=" ; Tbot
Locate 2 , 9 : Lcd "M="
If Moisture = 0 Then
Locate 2 , 11 : Lcd "H"       'select line 2 and position 12
End If
If Moisture = 255 Then
Locate 2 , 11 : Lcd "L"
End If

Locate 2 , 14 : Lcd "FAN=";
If Fan = 0 Then
Locate 2 , 18 : Lcd "OFF"       '
End If
If Fan = 255 Then
Locate 2 , 18 : Lcd "ON"       '
End If

Locate 3 , 1 : Lcd "OUT=" ; Tout
Locate 3 , 12 : Lcd "BATT=" ; Battd10 ; "." ; Battr

Locate 4 , 1 : Lcd "HTR=" ; Heater_on ; "-" ; Heateroffventshut
Locate 4 , 12 : Lcd "HEAT="
If Heat = 0 Then
Locate 4 , 17 : Lcd "OFF"
End If
If Heat = 255 Then
Locate 4 , 17 : Lcd "ON"
End If
Locate 4 , 20
Lcd Tx

Return

'****************************************************************************
'****************************************************************************

Openvents:       'FORWARD 4 STEPS
Set D2       '70.7% CURRENT TO THE STEPPER MOTOR
For N = 1 To 14
Set Pa : Set Pb
Waitms 20
'DL165   Greenhouse Controller

Reset Pa : Set Pb
Waitms 20
Reset Pa : Reset Pb
Waitms 20
Set Pa : Reset Pb
Waitms 20
Next N
Reset D2       'ZERO CURRENT TO THE STEPPER MOTOR

Set Phgear       'Phgear HI OPENs gear vent
Set D2gear       'D2gear HI turns on current
Wait 17       'wait 17 sec
Reset D2gear       'turn the current OFF

Vents = 255
Return

'****************************************************************************
'****************************************************************************

Shutvents:       'REVERSE 4 STEPS
Set D2       '70.7% CURRENT TO THE STEPPER MOTOR
For N = 1 To 14
Set Pa : Reset Pb
Waitms 20
Reset Pa : Reset Pb
Waitms 20
Reset Pa : Set Pb
Waitms 20
Set Pa : Set Pb
Waitms 20
Next N
Reset D2       'ZERO CURRENT TO THE STEPPER MOTOR

Reset Phgear       'Phgear LO SHUTs gear vent
Set D2gear       'D2gear HI turns on current
Wait 17       'wait 17 sec
Reset D2gear       'turn the current OFF

Vents = 0
Return

'****************************************************************************
'****************************************************************************
